@@ -133,15 +133,61 @@ DotWriter<Graph> dot_writer(const Graph& g)
 	return DotWriter<Graph>(g);
 }
 
+/** legal characters for an unquoted node ID */
+static const uint8_t GRAPHVIZ_ID_CHARS[256] = {
+	0, 0, 0, 0, 0, 0, 0, 0, // 000-007:
+	0, 0, 0, 0, 0, 0, 0, 0, // 008-015:
+	0, 0, 0, 0, 0, 0, 0, 0, // 016-023:
+	0, 0, 0, 0, 0, 0, 0, 0, // 024-031:
+	0, 0, 0, 0, 0, 0, 0, 0, // 032-039:    !, ", #, $, %, &, ',
+	0, 0, 0, 1, 0, 1, 1, 0, // 040-047: (, ), *, +, ,, -, ., /,
+	1, 1, 1, 1, 1, 1, 1, 1, // 048-055: 0, 1, 2, 3, 4, 5, 6, 7,
+	1, 1, 0, 0, 0, 0, 0, 0, // 056-063: 8, 9, :, ;, <, =, >, ?,
+	0, 1, 1, 1, 1, 1, 1, 1, // 064-071: @, A, B, C, D, E, F, G,
+	1, 1, 1, 1, 1, 1, 1, 1, // 072-079: H, I, J, K, L, M, N, O,
+	1, 1, 1, 1, 1, 1, 1, 1, // 080-087: P, Q, R, S, T, U, V, W,
+	1, 1, 1, 0, 0, 0, 0, 1, // 088-095: X, Y, Z, [, \, ], ^, _,
+	0, 1, 1, 1, 1, 1, 1, 1, // 096-103: `, a, b, c, d, e, f, g,
+	1, 1, 1, 1, 1, 1, 1, 1, // 104-111: h, i, j, k, l, m, n, o,
+	1, 1, 1, 1, 1, 1, 1, 1, // 112-119: p, q, r, s, t, u, v, w,
+	1, 1, 1, 0, 0, 0, 0, 0, // 120-127: x, y, z, {, |, }, ~,
+	0, 0, 0, 0, 0, 0, 0, 0, // 128-135:
+	0, 0, 0, 0, 0, 0, 0, 0, // 136-143:
+	0, 0, 0, 0, 0, 0, 0, 0, // 144-151:
+	0, 0, 0, 0, 0, 0, 0, 0, // 152-159:
+	0, 0, 0, 0, 0, 0, 0, 0, // 160-167:
+	0, 0, 0, 0, 0, 0, 0, 0, // 168-175:
+	0, 0, 0, 0, 0, 0, 0, 0, // 176-183:
+	0, 0, 0, 0, 0, 0, 0, 0, // 184-191:
+	0, 0, 0, 0, 0, 0, 0, 0, // 192-199:
+	0, 0, 0, 0, 0, 0, 0, 0, // 200-207:
+	0, 0, 0, 0, 0, 0, 0, 0, // 208-215:
+	0, 0, 0, 0, 0, 0, 0, 0, // 216-223:
+	0, 0, 0, 0, 0, 0, 0, 0, // 224-231:
+	0, 0, 0, 0, 0, 0, 0, 0, // 232-239:
+	0, 0, 0, 0, 0, 0, 0, 0, // 240-247:
+	0, 0, 0, 0, 0, 0, 0, 0  // 248-255:
+};
+
 /** Read a string delimited by double quotes. */
 static inline
 std::istream& read_dot_name(std::istream& in, std::string& s)
 {
-	if (in >> std::ws && in.peek() == '"') {
+	in >> std::ws;
+	if (in.peek() == '"') {
+		// quoted node ID
 		in.get();
 		getline(in, s, '"');
-	} else
+	} else if (GRAPHVIZ_ID_CHARS[in.peek()]) {
+		// unquoted node ID
+		std::ostringstream ss;
+		while (GRAPHVIZ_ID_CHARS[in.peek()])
+			ss << (char)in.get();
+		assert(ss.good());
+		s = ss.str();
+	} else {
 		in.clear(std::ios::failbit);
+	}
 	return in;
 }
 
@@ -170,12 +216,17 @@ std::istream& read_dot(std::istream& in, Graph& g, BetterEP betterEP)
 	typedef typename graph_traits<Graph>::edge_descriptor
 		edge_descriptor;
 	typedef typename edge_property<Graph>::type edge_property_type;
+	typedef typename graph_traits<Graph>::directed_category
+		directed_category;
+
+	bool isDirectedGraph = boost::detail::is_directed(directed_category());
 
 	// Add vertices if this graph is empty.
 	bool addVertices = num_vertices(g) == 0;
 
 	// Graph properties
-	in >> expect("digraph") >> Ignore('{');
+	in >> expect(isDirectedGraph ? "digraph" : "graph");
+	in >> Ignore('{');
 	assert(in);
 
 	edge_property_type defaultEdgeProp;
@@ -243,7 +294,7 @@ std::istream& read_dot(std::istream& in, Graph& g, BetterEP betterEP)
 			}
 		} else if (c == '-') {
 			// Edge
-			in >> expect(">");
+			in >> expect(isDirectedGraph ? ">" : "-");
 			assert(in);
 			g_contigNames.lock();
 
