@@ -17,14 +17,14 @@
 #include <cstdlib>
 #include <getopt.h>
 #include <iostream>
-#include <sstream>
 #include <stdint.h>
-#include <string>
 #include <utility>
 #include <queue>
 #if _OPENMP
 # include <omp.h>
 #endif
+#include "DataBase/Options.h"
+#include "DataBase/DB.h"
 
 using namespace std;
 using namespace boost;
@@ -32,6 +32,8 @@ using namespace boost::algorithm;
 
 #define PROGRAM "abyss-map"
 
+DB db;
+
 static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
@@ -58,15 +60,27 @@ static const char USAGE_MESSAGE[] =
 "      --no-multi          don't Align unaligned segments [default]\n"
 "      --SS                expect contigs to be oriented correctly\n"
 "      --no-SS             no assumption about contig orientation\n"
+"      --rc                map the sequence and its reverse complement [default]\n"
+"      --no-rc             do not map the reverse complement sequence\n"
+"  -a, --alphabet=STRING   use the alphabet STRING [-ACGT]\n"
+"      --alpha             equivalent to --no-rc -a' ABCDEFGHIJKLMNOPQRSTUVWXYZ'\n"
+"      --dna               equivalent to --rc    -a'-ACGT'\n"
+"      --protein           equivalent to --no-rc -a'#*ACDEFGHIKLMNPQRSTVWY'\n"
 "      --chastity          discard unchaste reads\n"
 "      --no-chastity       do not discard unchaste reads [default]\n"
 "  -v, --verbose           display verbose output\n"
 "      --help              display this help and exit\n"
 "      --version           output version information and exit\n"
+"      --db=FILE           specify path of database repository in FILE\n"
+"      --library=NAME      specify library NAME for database\n"
+"      --strain=NAME       specify strain NAME for database\n"
+"      --species=NAME      specify species NAME for database\n"
 "\n"
 "Report bugs to <" PACKAGE_BUGREPORT ">.\n";
 
 namespace opt {
+	string db;
+	dbVars metaVars;
 	/** Find matches at least k bp. */
 	static unsigned k;
 
@@ -79,6 +93,12 @@ namespace opt {
 	/** Run a strand-specific RNA-Seq alignments. */
 	static int ss;
 
+	/** Do not map the sequence's reverse complement. */
+	static int norc;
+
+	/** The alphabet. */
+	static string alphabet = "-ACGT";
+
 	/** Identify duplicate and subsumed sequences. */
 	static bool dup = false;
 
@@ -92,9 +112,15 @@ namespace opt {
 	static int verbose;
 }
 
+// for sqlite params
+static bool haveDbParam(false);
+
 static const char shortopts[] = "j:k:l:s:dv";
 
-enum { OPT_HELP = 1, OPT_VERSION };
+enum { OPT_HELP = 1, OPT_VERSION,
+	OPT_ALPHA, OPT_DNA, OPT_PROTEIN,
+	OPT_DB, OPT_LIBRARY, OPT_STRAIN, OPT_SPECIES,
+};
 
 static const struct option longopts[] = {
 	{ "sample", required_argument, NULL, 's' },
@@ -107,11 +133,22 @@ static const struct option longopts[] = {
 	{ "no-multi", no_argument, &opt::multi, 0 },
 	{ "SS", no_argument, &opt::ss, 1 },
 	{ "no-SS", no_argument, &opt::ss, 0 },
+	{ "rc", no_argument, &opt::norc, 0 },
+	{ "no-rc", no_argument, &opt::norc, 1 },
+	{ "alphabet", optional_argument, NULL, 'a' },
+	{ "alpha", optional_argument, NULL, OPT_ALPHA },
+	{ "dna", optional_argument, NULL, OPT_DNA },
+	{ "protein", optional_argument, NULL, OPT_PROTEIN },
+	{ "decompress", no_argument, NULL, 'd' },
 	{ "verbose", no_argument, NULL, 'v' },
 	{ "chastity", no_argument, &opt::chastityFilter, 1 },
 	{ "no-chastity", no_argument, &opt::chastityFilter, 0 },
 	{ "help", no_argument, NULL, OPT_HELP },
 	{ "version", no_argument, NULL, OPT_VERSION },
+	{ "db", required_argument, NULL, OPT_DB },
+	{ "library", required_argument, NULL, OPT_LIBRARY },
+	{ "strain", required_argument, NULL, OPT_STRAIN },
+	{ "species", required_argument, NULL, OPT_SPECIES },
 	{ NULL, 0, NULL, 0 }
 };
 
@@ -244,7 +281,7 @@ static void printDuplicates(const Match& m, const Match& rcm,
 {
 	size_t myLen = m.qspan();
 	size_t maxLen;
-	if (opt::ss)
+	if (opt::ss || opt::norc)
 		maxLen = getMaxLen(m, faIndex, fmIndex);
 	else
 		maxLen = max(getMaxLen(m, faIndex, fmIndex),
@@ -261,7 +298,7 @@ static void printDuplicates(const Match& m, const Match& rcm,
 	}
 	size_t myPos = getMyPos(m, faIndex, fmIndex, rec.id);
 	size_t minPos;
-	if (opt::ss)
+	if (opt::ss || opt::norc)
 		minPos = getMinPos(m, maxLen, faIndex, fmIndex);
 	else
 		minPos = min(getMinPos(m, maxLen, faIndex, fmIndex),
@@ -284,6 +321,8 @@ pair<Match, Match> findMatch(const FMIndex& fmIndex, const string& seq)
 {
 	Match m = fmIndex.find(seq,
 			opt::dup ? seq.length() : opt::k);
+	if (opt::norc)
+		return make_pair(m, Match());
 
 	string rcqseq = reverseComplement(seq);
 	Match rcm;
@@ -460,7 +499,7 @@ static void buildFMIndex(FMIndex& fm, const char* path)
 	}
 
 	transform(s.begin(), s.end(), s.begin(), ::toupper);
-	fm.setAlphabet("-ACGT");
+	fm.setAlphabet(opt::alphabet);
 	fm.assign(s.begin(), s.end());
 }
 
@@ -511,6 +550,9 @@ int main(int argc, char** argv)
 	opt::chastityFilter = false;
 	opt::trimMasked = false;
 
+	if (!opt::db.empty())
+		opt::metaVars.resize(3);
+
 	bool die = false;
 	for (int c; (c = getopt_long(argc, argv,
 					shortopts, longopts, NULL)) != -1;) {
@@ -523,6 +565,22 @@ int main(int argc, char** argv)
 				break;
 			case 's': arg >> opt::sampleSA; break;
 			case 'd': opt::dup = true; break;
+			case 'a':
+				opt::alphabet = arg.str();
+				arg.clear(ios::eofbit);
+				break;
+			case OPT_ALPHA:
+				opt::alphabet = " ABCDEFGHIJKLMNOPQRSTUVWXYZ";
+				opt::norc = true;
+				break;
+			case OPT_DNA:
+				opt::alphabet = "-ACGT";
+				opt::norc = false;
+				break;
+			case OPT_PROTEIN:
+				opt::alphabet = "#*ACDEFGHIKLMNPQRSTVWY";
+				opt::norc = true;
+				break;
 			case 'v': opt::verbose++; break;
 			case OPT_HELP:
 				cout << USAGE_MESSAGE;
@@ -530,6 +588,20 @@ int main(int argc, char** argv)
 			case OPT_VERSION:
 				cout << VERSION_MESSAGE;
 				exit(EXIT_SUCCESS);
+			case OPT_DB:
+				arg >> opt::db;
+				haveDbParam = true;
+				break;
+			case OPT_LIBRARY:
+				arg >> opt::metaVars[0];
+				haveDbParam = true;
+				break;
+			case OPT_STRAIN:
+				arg >> opt::metaVars[1];
+				haveDbParam = true;
+				break;
+			case OPT_SPECIES:
+				arg >> opt::metaVars[2]; break;
 		}
 		if (optarg != NULL && !arg.eof()) {
 			cerr << PROGRAM ": invalid option: `-"
@@ -565,6 +637,16 @@ int main(int argc, char** argv)
 		omp_set_num_threads(opt::threads);
 #endif
 
+	if (!opt::db.empty()) {
+		init(db, opt::db,
+				opt::verbose,
+				PROGRAM,
+				opt::getCommand(argc, argv),
+				opt::metaVars);
+		addToDb(db, "K", opt::k);
+		addToDb(db, "SS", opt::ss);
+	}
+
 	const char* targetFile(argv[--argc]);
 	ostringstream ss;
 	ss << targetFile << ".fm";
@@ -622,6 +704,9 @@ int main(int argc, char** argv)
 				<< setprecision(3) << (float)bytes / bp << " B/bp.\n";
 	}
 
+	if (!opt::db.empty())
+		addToDb(db, "readContigs", faIndex.size());
+
 	// Check that the indexes are up to date.
 	checkIndexes(targetFile, fmIndex, faIndex);
 
@@ -649,13 +734,23 @@ int main(int argc, char** argv)
 			<< "Mapped " << unique << " of " << total
 			<< " reads uniquely (" << (float)100 * unique / total
 			<< "%)\n";
-		if (opt::ss)
+
+		// TODO: This block shouldn't be in a verbose restricted section.
+		if (!opt::db.empty()) {
+			addToDb(db, "read_alignments_initial", total);
+			addToDb(db, "mapped", mapped);
+			addToDb(db, "mapped_uniq", unique);
+			addToDb(db, "reads_map_ss", g_count.suboptimal);
+		}
+
+		if (opt::ss) {
 			cerr << "Mapped " << g_count.suboptimal
 				<< " (" << (float)100 * g_count.suboptimal / total << "%)"
 				<< " reads to the opposite strand of the optimal mapping.\n"
 				<< "Made " << g_count.subunmapped << " ("
 				<< (float)100 * g_count.subunmapped / total << "%)"
 				<< " unmapped suboptimal decisions.\n";
+		}
 	}
 
 	cout.flush();
