@@ -1,11 +1,11 @@
 #include "NetworkSequenceCollection.h"
+#include "Assembly/AssemblyAlgorithms.h"
 #include "Assembly/Options.h"
-#include "AssemblyAlgorithms.h"
+#include "Common/Histogram.h"
+#include "Common/Log.h"
 #include "Common/Options.h"
-#include "FastaWriter.h"
-#include "Histogram.h"
-#include "Log.h"
-#include "StringUtil.h"
+#include "Common/StringUtil.h"
+#include "DataLayer/FastaWriter.h"
 #include <climits> // for UINT_MAX
 #include <cstdlib>
 #include <fstream>
@@ -14,6 +14,17 @@
 
 using namespace std;
 
+namespace NSC
+{
+	dbMap moveFromAaStatMap()
+	{
+		dbMap temp;
+		temp.insert(AssemblyAlgorithms::tempStatMap.getAC());
+		AssemblyAlgorithms::tempStatMap.clear();
+		return temp;
+	}
+}
+
 // Don't load data into the control process when we have at least
 // DEDICATE_CONTROL_AT total processes. This is needed because the
 // control node uses a lot of memory at large NP.
@@ -134,7 +145,7 @@ void NetworkSequenceCollection::run()
 			{
 				assert(m_trimStep != 0);
 				m_comm.barrier();
-				size_t numRemoved = performNetworkTrim(this);
+				size_t numRemoved = performNetworkTrim();
 				EndState();
 				SetState(NAS_WAITING);
 				m_comm.sendCheckPointMessage(numRemoved);
@@ -155,7 +166,7 @@ void NetworkSequenceCollection::run()
 				m_comm.reduce(m_data.cleanup());
 				m_lowCoverageContigs = 0;
 				m_lowCoverageKmer = 0;
-				numAssembled = performNetworkAssembly(this);
+				numAssembled = performNetworkAssembly();
 				EndState();
 				SetState(NAS_WAITING);
 				m_comm.sendCheckPointMessage();
@@ -176,7 +187,7 @@ void NetworkSequenceCollection::run()
 			case NAS_DISCOVER_BUBBLES:
 			{
 				size_t numDiscovered
-					= performNetworkDiscoverBubbles(this);
+					= performNetworkDiscoverBubbles();
 				EndState();
 				SetState(NAS_WAITING);
 				m_comm.sendCheckPointMessage(numDiscovered);
@@ -230,7 +241,7 @@ void NetworkSequenceCollection::run()
 				m_comm.barrier();
 				pumpNetwork();
 				FastaWriter writer(opt::contigsTempPath.c_str());
-				numAssembled = performNetworkAssembly(this, &writer);
+				numAssembled = performNetworkAssembly(&writer);
 				EndState();
 				SetState(NAS_WAITING);
 				m_comm.sendCheckPointMessage();
@@ -283,7 +294,6 @@ size_t NetworkSequenceCollection::controlErode()
 	numEroded += m_comm.reduce(
 			AssemblyAlgorithms::getNumEroded());
 	cout << "Eroded " << numEroded << " tips.\n";
-
 	size_t removed = m_comm.reduce(m_data.cleanup());
 	m_comm.barrier();
 	assert(removed == numEroded);
@@ -322,7 +332,7 @@ size_t NetworkSequenceCollection::controlTrimRound(unsigned trimLen)
 	SetState(NAS_TRIM);
 	m_comm.sendControlMessage(APC_TRIM, trimLen);
 	m_comm.barrier();
-	size_t numRemoved = performNetworkTrim(this);
+	size_t numRemoved = performNetworkTrim();
 	EndState();
 
 	m_numReachedCheckpoint++;
@@ -339,7 +349,7 @@ size_t NetworkSequenceCollection::controlTrimRound(unsigned trimLen)
 }
 
 /** Perform multiple rounds of trimming until complete. */
-void NetworkSequenceCollection::controlTrim(unsigned start)
+void NetworkSequenceCollection::controlTrim(unsigned& sum, unsigned start)
 {
 	if (opt::trimLen == 0)
 		return;
@@ -355,6 +365,7 @@ void NetworkSequenceCollection::controlTrim(unsigned start)
 	}
 	cout << "Pruned " << total << " tips in "
 		<< rounds << " rounds.\n";
+	sum += total;
 }
 
 /** Remove low-coverage contigs. */
@@ -375,7 +386,7 @@ void NetworkSequenceCollection::controlCoverage()
 	m_lowCoverageContigs = 0;
 	m_lowCoverageKmer = 0;
 	pair<size_t, size_t> numAssembled
-		= performNetworkAssembly(this);
+		= performNetworkAssembly();
 	EndState();
 
 	m_numReachedCheckpoint++;
@@ -398,6 +409,12 @@ void NetworkSequenceCollection::controlCoverage()
 	size_t lowCoverageKmer = m_comm.reduce(m_lowCoverageKmer);
 	cout << "Removed " << lowCoverageKmer << " k-mer in "
 		<< lowCoverageContigs << " low-coverage contigs.\n";
+
+	if (!opt::db.empty()) {
+		AssemblyAlgorithms::addToDb ("totalLowCovCntg", lowCoverageContigs);
+		AssemblyAlgorithms::addToDb ("totalLowCovKmer", lowCoverageKmer);
+	}
+
 	EndState();
 
 	SetState(NAS_SPLIT_AMBIGUOUS);
@@ -418,7 +435,11 @@ void NetworkSequenceCollection::controlCoverage()
 /** Run the assembly state machine for the controller (rank = 0). */
 void NetworkSequenceCollection::runControl()
 {
+	unsigned prunedSum = 0;
+	unsigned erosionSum = 0;
+	unsigned finalAmbg = 0;
 	SetState(NAS_LOADING);
+	size_t temp;
 	while (m_state != NAS_DONE) {
 		switch (m_state) {
 			case NAS_LOADING:
@@ -446,6 +467,9 @@ void NetworkSequenceCollection::runControl()
 					<< toSI(numLoaded * sizeof (value_type))
 					<< "B of RAM is required.\n";
 
+				if (!opt::db.empty())
+					AssemblyAlgorithms::addToDb("loadedKmer", numLoaded);
+
 				Histogram myh
 					= AssemblyAlgorithms::coverageHistogram(m_data);
 				Histogram h(m_comm.reduce(myh.toVector()));
@@ -473,18 +497,24 @@ void NetworkSequenceCollection::runControl()
 						NAS_ADJ_COMPLETE);
 				m_comm.barrier();
 				pumpNetwork();
+				temp = m_comm.reduce(m_numBasesAdjSet);
 				logger(0) << "Added " << m_numBasesAdjSet
 					<< " edges.\n";
-				cout << "Added " << m_comm.reduce(m_numBasesAdjSet)
-					<< " edges.\n";
-				EndState();
+				cout << "Added " << temp << " edges.\n";
 
+				if (!opt::db.empty())
+					AssemblyAlgorithms::addToDb ("EdgesGenerated", temp);
+
+				EndState();
 				SetState(opt::erode > 0 ? NAS_ERODE : NAS_TRIM);
 				break;
 			case NAS_ERODE:
 				assert(opt::erode > 0);
 				cout << "Eroding tips...\n";
-				controlErode();
+
+				erosionSum += controlErode();
+				//controlErode();
+
 				SetState(NAS_TRIM);
 				break;
 
@@ -504,7 +534,7 @@ void NetworkSequenceCollection::runControl()
 				exit(EXIT_FAILURE);
 
 			case NAS_TRIM:
-				controlTrim();
+				controlTrim(prunedSum);
 				SetState(opt::coverage > 0 ? NAS_COVERAGE
 						: opt::bubbleLen > 0 ? NAS_POPBUBBLE
 						: NAS_MARK_AMBIGUOUS);
@@ -528,11 +558,18 @@ void NetworkSequenceCollection::runControl()
 				out.close();
 				cout << "Removed " << numPopped << " bubbles.\n";
 
+				if (!opt::db.empty())
+					AssemblyAlgorithms::addToDb ("poppedBubbles", numPopped);
+
 				SetState(NAS_MARK_AMBIGUOUS);
 				break;
 			}
 			case NAS_MARK_AMBIGUOUS:
-				controlMarkAmbiguous();
+
+				finalAmbg = controlMarkAmbiguous();
+
+				//controlMarkAmbiguous();
+
 				SetState(NAS_ASSEMBLE);
 				break;
 			case NAS_ASSEMBLE:
@@ -543,7 +580,7 @@ void NetworkSequenceCollection::runControl()
 				pumpNetwork();
 				FastaWriter writer(opt::contigsTempPath.c_str());
 				pair<size_t, size_t> numAssembled
-					= performNetworkAssembly(this, &writer);
+					= performNetworkAssembly(&writer);
 				EndState();
 
 				m_numReachedCheckpoint++;
@@ -562,6 +599,11 @@ void NetworkSequenceCollection::runControl()
 					<< " k-mer in " << numAssembled.first
 					<< " contigs.\n";
 
+				if (!opt::db.empty()) {
+					AssemblyAlgorithms::addToDb ("assembledKmerNum", numAssembled.second);
+					AssemblyAlgorithms::addToDb ("assembledCntg", numAssembled.first);
+				}
+
 				SetState(NAS_DONE);
 				break;
 			}
@@ -569,6 +611,13 @@ void NetworkSequenceCollection::runControl()
 				break;
 		}
 	}
+
+	if (!opt::db.empty()) {
+		AssemblyAlgorithms::addToDb ("finalAmbgVertices", finalAmbg);
+		AssemblyAlgorithms::addToDb ("totalErodedTips", erosionSum);
+		AssemblyAlgorithms::addToDb ("totalPrunedTips", prunedSum);
+	}
+
 }
 
 void NetworkSequenceCollection::EndState()
@@ -633,7 +682,7 @@ size_t NetworkSequenceCollection::pumpNetwork()
 }
 
 /** Call the observers of the specified sequence. */
-void NetworkSequenceCollection::notify(const Kmer& key)
+void NetworkSequenceCollection::notify(const V& key)
 {
 	switch (m_state) {
 		case NAS_ERODE:
@@ -731,9 +780,9 @@ void NetworkSequenceCollection::parseControlMessage(int source)
 void NetworkSequenceCollection::handle(
 		int senderID, const SeqDataRequest& message)
 {
-	const Kmer& kmer = message.m_seq;
+	const V& kmer = message.m_seq;
 	assert(isLocal(kmer));
-	ExtensionRecord extRec;
+	SymbolSetPair extRec;
 	int multiplicity = -1;
 	bool found = m_data.getSeqData(kmer, extRec, multiplicity);
 	assert(found);
@@ -752,16 +801,16 @@ void NetworkSequenceCollection::handle(
 }
 
 /** Distributed trimming function. */
-size_t NetworkSequenceCollection::performNetworkTrim(
-		ISequenceCollection* seqCollection)
+size_t NetworkSequenceCollection::performNetworkTrim()
 {
 	Timer timer("NetworkTrim");
+	NetworkSequenceCollection* seqCollection = this;
 	size_t numBranchesRemoved = 0;
 
 	// The branch ids
 	uint64_t branchGroupID = 0;
 
-	for (ISequenceCollection::iterator iter = seqCollection->begin();
+	for (iterator iter = seqCollection->begin();
 			iter != seqCollection->end(); ++iter) {
 		if (iter->second.deleted())
 			continue;
@@ -848,8 +897,9 @@ size_t NetworkSequenceCollection::processBranchesTrim()
 
 /** Discover bubbles to pop. */
 size_t NetworkSequenceCollection::
-performNetworkDiscoverBubbles(ISequenceCollection* seqCollection)
+performNetworkDiscoverBubbles()
 {
+	NetworkSequenceCollection* seqCollection = this;
 	Timer timer("NetworkDiscoverBubbles");
 
 	// The branch ids
@@ -864,7 +914,7 @@ performNetworkDiscoverBubbles(ISequenceCollection* seqCollection)
 	// Set the cutoffs
 	const unsigned maxNumBranches = 3;
 
-	for (ISequenceCollection::iterator iter = seqCollection->begin();
+	for (iterator iter = seqCollection->begin();
 			iter != seqCollection->end(); ++iter) {
 		if (iter->second.deleted())
 			continue;
@@ -872,7 +922,7 @@ performNetworkDiscoverBubbles(ISequenceCollection* seqCollection)
 		if (++count % 100000 == 0)
 			logger(1) << "Popping bubbles: " << count << '\n';
 
-		ExtensionRecord extRec = iter->second.extension();
+		SymbolSetPair extRec = iter->second.extension();
 		for (extDirection dir = SENSE; dir <= ANTISENSE; ++dir) {
 			if (extRec.dir[dir].isAmbiguous()) {
 				BranchGroupMap::iterator groupIter
@@ -991,7 +1041,7 @@ size_t NetworkSequenceCollection::controlDiscoverBubbles()
 	SetState(NAS_DISCOVER_BUBBLES);
 	m_comm.sendControlMessage(APC_SET_STATE, NAS_DISCOVER_BUBBLES);
 
-	size_t numDiscovered = performNetworkDiscoverBubbles(this);
+	size_t numDiscovered = performNetworkDiscoverBubbles();
 	EndState();
 
 	m_numReachedCheckpoint++;
@@ -1066,16 +1116,20 @@ size_t NetworkSequenceCollection::controlSplitAmbiguous()
 	while (!checkpointReached())
 		pumpNetwork();
 	cout << "Split " << m_checkpointSum << " ambiguous branches.\n";
+
+	if (!opt::db.empty())
+		AssemblyAlgorithms::addToDb ("totalSplitAmbg", m_checkpointSum);
+
 	return m_checkpointSum;
 }
 
 /** Assemble a contig. */
 void NetworkSequenceCollection::assembleContig(
-		ISequenceCollection* seqCollection, FastaWriter* writer,
+		FastaWriter* writer,
 		BranchRecord& branch, unsigned id)
 {
 	size_t removed = AssemblyAlgorithms::assembleContig(
-			seqCollection, writer, branch, id);
+			this, writer, branch, id);
 	if (removed > 0) {
 		m_lowCoverageContigs++;
 		m_lowCoverageKmer += removed;
@@ -1097,15 +1151,15 @@ namespace std {
  * @return the number of contigs and k-mer assembled
  */
 pair<size_t, size_t> NetworkSequenceCollection::
-performNetworkAssembly(ISequenceCollection* seqCollection,
-		FastaWriter* fileWriter)
+performNetworkAssembly(FastaWriter* fileWriter)
 {
 	Timer timer("NetworkAssembly");
+	NetworkSequenceCollection* seqCollection = this;
 	pair<size_t, size_t> numAssembled(0, 0);
 	uint64_t branchGroupID = 0;
 	assert(m_activeBranchGroups.empty());
 
-	for (ISequenceCollection::iterator iter = seqCollection->begin();
+	for (iterator iter = seqCollection->begin();
 			iter != seqCollection->end(); ++iter) {
 		if (iter->second.deleted())
 			continue;
@@ -1123,7 +1177,7 @@ performNetworkAssembly(ISequenceCollection* seqCollection,
 			BranchRecord currBranch(SENSE);
 			currBranch.push_back(*iter);
 			currBranch.terminate(BS_NOEXT);
-			assembleContig(seqCollection, fileWriter, currBranch,
+			assembleContig(fileWriter, currBranch,
 					m_numAssembled + numAssembled.first);
 			numAssembled.first++;
 			numAssembled.second += currBranch.size();
@@ -1140,13 +1194,13 @@ performNetworkAssembly(ISequenceCollection* seqCollection,
 		// Generate the first extension request
 		BranchRecord& branch = inserted.first->second[0];
 		branch.push_back(*iter);
-		Kmer kmer = iter->first;
+		V kmer = iter->first;
 		AssemblyAlgorithms::extendBranch(branch,
 				kmer, iter->second.getExtension(dir));
 		assert(branch.isActive());
 		generateExtensionRequest(branchGroupID++, 0, kmer);
 
-		numAssembled += processBranchesAssembly(seqCollection,
+		numAssembled += processBranchesAssembly(
 				fileWriter, numAssembled.first);
 		seqCollection->pumpNetwork();
 
@@ -1155,7 +1209,7 @@ performNetworkAssembly(ISequenceCollection* seqCollection,
 			while(m_activeBranchGroups.size() > LOW_ACTIVE)
 			{
 				seqCollection->pumpNetwork();
-				numAssembled += processBranchesAssembly(seqCollection,
+				numAssembled += processBranchesAssembly(
 						fileWriter, numAssembled.first);
 			}
 		}
@@ -1164,7 +1218,7 @@ performNetworkAssembly(ISequenceCollection* seqCollection,
 	// Clear out the remaining branches
 	while(!m_activeBranchGroups.empty())
 	{
-		numAssembled += processBranchesAssembly(seqCollection,
+		numAssembled += processBranchesAssembly(
 				fileWriter, numAssembled.first);
 		seqCollection->pumpNetwork();
 	}
@@ -1186,8 +1240,7 @@ performNetworkAssembly(ISequenceCollection* seqCollection,
  * @return the number of contigs and k-mer assembled
  */
 pair<size_t, size_t> NetworkSequenceCollection::
-processBranchesAssembly(ISequenceCollection* seqCollection,
-		FastaWriter* fileWriter, unsigned currContigID)
+processBranchesAssembly(FastaWriter* fileWriter, unsigned currContigID)
 {
 	size_t assembledContigs = 0, assembledKmer = 0;
 	for (BranchGroupMap::iterator it = m_activeBranchGroups.begin();
@@ -1202,7 +1255,7 @@ processBranchesAssembly(ISequenceCollection* seqCollection,
 					|| (!opt::ss && branch.isCanonical())) {
 				assembledContigs++;
 				assembledKmer += branch.size();
-				assembleContig(seqCollection, fileWriter, branch,
+				assembleContig(fileWriter, branch,
 						m_numAssembled + currContigID++);
 			}
 			m_activeBranchGroups.erase(it++);
@@ -1214,10 +1267,10 @@ processBranchesAssembly(ISequenceCollection* seqCollection,
 
 /** Send a request for the edges of vertex kmer. */
 void NetworkSequenceCollection::generateExtensionRequest(
-		uint64_t groupID, uint64_t branchID, const Kmer& kmer)
+		uint64_t groupID, uint64_t branchID, const V& kmer)
 {
 	if (isLocal(kmer)) {
-		ExtensionRecord extRec;
+		SymbolSetPair extRec;
 		int multiplicity = -1;
 		bool success = m_data.getSeqData(kmer, extRec, multiplicity);
 		assert(success);
@@ -1248,8 +1301,8 @@ void NetworkSequenceCollection::generateExtensionRequests(
 }
 
 void NetworkSequenceCollection::processSequenceExtension(
-		uint64_t groupID, uint64_t branchID, const Kmer& seq,
-		const ExtensionRecord& extRec, int multiplicity)
+		uint64_t groupID, uint64_t branchID, const V& seq,
+		const SymbolSetPair& extRec, int multiplicity)
 {
 	switch(m_state)
 	{
@@ -1285,14 +1338,14 @@ void NetworkSequenceCollection::processSequenceExtension(
 
 /** Process a sequence extension for trimming. */
 void NetworkSequenceCollection::processLinearSequenceExtension(
-		uint64_t groupID, uint64_t branchID, const Kmer& seq,
-		const ExtensionRecord& extRec, int multiplicity,
+		uint64_t groupID, uint64_t branchID, const V& seq,
+		const SymbolSetPair& extRec, int multiplicity,
 		unsigned maxLength)
 {
 	BranchGroupMap::iterator iter
 		= m_activeBranchGroups.find(groupID);
 	assert(iter != m_activeBranchGroups.end());
-	Kmer currSeq = seq;
+	V currSeq = seq;
 	bool active = AssemblyAlgorithms::processLinearExtensionForBranch(
 			iter->second[branchID], currSeq, extRec, multiplicity,
 			maxLength);
@@ -1302,8 +1355,8 @@ void NetworkSequenceCollection::processLinearSequenceExtension(
 
 /** Process a sequence extension for popping. */
 void NetworkSequenceCollection::processSequenceExtensionPop(
-		uint64_t groupID, uint64_t branchID, const Kmer& seq,
-		const ExtensionRecord& extRec, int multiplicity,
+		uint64_t groupID, uint64_t branchID, const V& seq,
+		const SymbolSetPair& extRec, int multiplicity,
 		unsigned maxLength)
 {
 	BranchGroupMap::iterator groupIt
@@ -1324,7 +1377,7 @@ void NetworkSequenceCollection::processSequenceExtensionPop(
 }
 
 /** Add a k-mer to this collection. */
-void NetworkSequenceCollection::add(const Kmer& seq,
+void NetworkSequenceCollection::add(const V& seq,
 		unsigned coverage)
 {
 	if (isLocal(seq)) {
@@ -1336,7 +1389,7 @@ void NetworkSequenceCollection::add(const Kmer& seq,
 }
 
 /** Remove a k-mer from this collection. */
-void NetworkSequenceCollection::remove(const Kmer& seq)
+void NetworkSequenceCollection::remove(const V& seq)
 {
 	if (isLocal(seq))
 		m_data.remove(seq);
@@ -1355,7 +1408,7 @@ checkpointReached(unsigned numRequired) const
 	return m_numReachedCheckpoint == numRequired;
 }
 
-void NetworkSequenceCollection::setFlag(const Kmer& seq, SeqFlag flag)
+void NetworkSequenceCollection::setFlag(const V& seq, SeqFlag flag)
 {
 	if (isLocal(seq))
 		m_data.setFlag(seq, flag);
@@ -1364,7 +1417,7 @@ void NetworkSequenceCollection::setFlag(const Kmer& seq, SeqFlag flag)
 }
 
 bool NetworkSequenceCollection::setBaseExtension(
-		const Kmer& seq, extDirection dir, uint8_t base)
+		const V& seq, extDirection dir, Symbol base)
 {
 	if (isLocal(seq)) {
 		if (m_data.setBaseExtension(seq, dir, base))
@@ -1380,7 +1433,7 @@ bool NetworkSequenceCollection::setBaseExtension(
 
 /** Remove the specified extensions from this k-mer. */
 void NetworkSequenceCollection::removeExtension(
-		const Kmer& seq, extDirection dir, SeqExt ext)
+		const V& seq, extDirection dir, SymbolSet ext)
 {
 	if (isLocal(seq)) {
 		m_data.removeExtension(seq, dir, ext);
@@ -1392,13 +1445,13 @@ void NetworkSequenceCollection::removeExtension(
 }
 
 /** Return whether this sequence belongs to this process. */
-bool NetworkSequenceCollection::isLocal(const Kmer& seq) const
+bool NetworkSequenceCollection::isLocal(const V& seq) const
 {
 	return computeNodeID(seq) == opt::rank;
 }
 
 /** Return the process ID to which the specified kmer belongs. */
-int NetworkSequenceCollection::computeNodeID(const Kmer& seq) const
+int NetworkSequenceCollection::computeNodeID(const V& seq) const
 {
 	if (opt::numProc < DEDICATE_CONTROL_AT) {
 		return seq.getCode() % (unsigned)opt::numProc;
