@@ -1,11 +1,11 @@
 #include "config.h"
-#include "Log.h"
 #include "NetworkSequenceCollection.h"
 #include "Assembly/Options.h"
+#include "Common/Log.h"
 #include "Common/Options.h"
+#include "Common/Timer.h"
+#include "Common/Uncompress.h"
 #include "DataLayer/FastaReader.h"
-#include "Timer.h"
-#include "Uncompress.h"
 #include <cerrno>
 #include <climits> // for HOST_NAME_MAX
 #include <cstdio> // for setvbuf
@@ -13,14 +13,19 @@
 #include <cstring> // for strerror
 #include <iostream>
 #include <mpi.h>
-#include <sstream>
 #include <unistd.h> // for gethostname
 #include <vector>
+#include "DataBase/DB.h"
 
 using namespace std;
 
 static const char* FASTA_SUFFIX = ".fa";
 
+#if PAIRED_DBG
+// Define KmerPair::s_length
+# include "PairedDBG/KmerPair.cc"
+#endif
+
 static void mergeFastaFiles(const string& outputPath, const string& inputPathPrefix, bool generateNewIds = false)
 {
 	cout << "Concatenating fasta files to " << outputPath << endl;
@@ -77,6 +82,9 @@ int main(int argc, char** argv)
 	// reinitialize uncompress.
 	uncompress_init();
 
+#if PAIRED_DBG
+	opt::singleKmerSize = -1;
+#endif
 	opt::parse(argc, argv);
 	if (opt::rank == 0)
 		cout << "Running on " << opt::numProc << " processors\n";
@@ -87,6 +95,13 @@ int main(int argc, char** argv)
 	logger(0) << "Running on host " << hostname << endl;
 	MPI_Barrier(MPI_COMM_WORLD);
 
+#if PAIRED_DBG
+	Kmer::setLength(opt::singleKmerSize);
+	KmerPair::setLength(opt::kmerSize);
+#else
+	Kmer::setLength(opt::kmerSize);
+#endif
+
 	if (opt::rank == 0) {
 		NetworkSequenceCollection networkSeqs;
 		networkSeqs.runControl();
@@ -103,6 +118,20 @@ int main(int argc, char** argv)
 		if (!opt::snpPath.empty())
 			mergeFastaFiles(opt::snpPath, "snp-");
 		cout << "Done." << endl;
+		DB db;
+		if (!opt::db.empty()) {
+			init(db,
+					opt::getUvalue(),
+					opt::getVvalue(),
+					"ABYSS-P",
+					opt::getCommand(),
+					opt::getMetaValue()
+			);
+			addToDb(db, "SS", opt::ss);
+			addToDb(db, "K", opt::kmerSize);
+			addToDb(db, "numProc", opt::numProc);
+			addToDb(db, NSC::moveFromAaStatMap());
+		}
 	}
 
 	return 0;
