@@ -75,7 +75,13 @@ static const char USAGE_MESSAGE[] =
 "      --SS              expect contigs to be oriented correctly\n"
 "      --no-SS           no assumption about contig orientation [default]\n"
 "  -g, --graph=FILE      write the contig adjacency graph to FILE\n"
-"      --dot             output bubbles in dot format\n"
+"      --adj             output the graph in ADJ format [default]\n"
+"      --asqg            output the graph in ASQG format\n"
+"      --dot             output the graph in GraphViz format\n"
+"      --gv              output the graph in GraphViz format\n"
+"      --gfa             output the graph in GFA format\n"
+"      --sam             output the graph in SAM format\n"
+"      --bubble-graph    output a graph of the bubbles\n"
 "  -j, --threads=N       use N parallel threads [1]\n"
 "  -v, --verbose         display verbose output\n"
 "      --help            display this help and exit\n"
@@ -104,8 +110,8 @@ namespace opt {
 	/** Write the contig adjacency graph to this file. */
 	static string graphPath;
 
-	/** Output bubbles in dot format. */
-	static int dot;
+	/** Output a graph of the bubbles. */
+	static int bubbleGraph;
 
 	int format; // used by ContigProperties
 
@@ -124,8 +130,14 @@ static const struct option longopts[] = {
 	{ "branches",      required_argument, NULL, 'a' },
 	{ "bubble-length", required_argument, NULL, 'b' },
 	{ "coverage",      required_argument, NULL, 'c' },
-	{ "dot",           no_argument,       &opt::dot, 1, },
+	{ "bubble-graph",  no_argument,       &opt::bubbleGraph, 1, },
 	{ "graph",         required_argument, NULL, 'g' },
+	{ "adj",           no_argument,       &opt::format, ADJ },
+	{ "asqg",          no_argument,       &opt::format, ASQG },
+	{ "dot",           no_argument,       &opt::format, DOT },
+	{ "gv",            no_argument,       &opt::format, DOT },
+	{ "gfa",           no_argument,       &opt::format, GFA },
+	{ "sam",           no_argument,       &opt::format, SAM },
 	{ "kmer",          required_argument, NULL, 'k' },
 	{ "identity",      required_argument, NULL, 'p' },
 	{ "scaffold",      no_argument,       &opt::scaffold, 1},
@@ -178,7 +190,7 @@ static void popBubble(Graph& g,
 		adj = g.adjacent_vertices(v);
 	copy(adj.first, adj.second, sorted.begin());
 	sort(sorted.begin(), sorted.end(), CompareCoverage(g));
-	if (opt::dot)
+	if (opt::bubbleGraph)
 #pragma omp critical(cout)
 	{
 		cout << '"' << get(vertex_name, g, v) << "\" -> {";
@@ -235,15 +247,15 @@ static float getAlignmentIdentity(const Graph& g,
 	unsigned nbranches = distance(first, last);
 	vector<int> inDists(nbranches);
 	transform(first, last, inDists.begin(),
-			bind(getDistance, boost::cref(g), t, _1));
+			boost::lambda::bind(getDistance, boost::cref(g), t, _1));
 	vector<int> outDists(nbranches);
 	transform(first, last, outDists.begin(),
-			bind(getDistance, boost::cref(g), _1, v));
+			boost::lambda::bind(getDistance, boost::cref(g), _1, v));
 	vector<int> insertLens(nbranches);
 	transform(first, last, insertLens.begin(),
-			bind(getDistance, boost::cref(g), t, _1)
-				+ bind(getLength, &g, _1)
-				+ bind(getDistance, boost::cref(g), _1, v));
+			boost::lambda::bind(getDistance, boost::cref(g), t, _1)
+				+ boost::lambda::bind(getLength, &g, _1)
+				+ boost::lambda::bind(getDistance, boost::cref(g), _1, v));
 
 	int max_in_overlap = -(*min_element(inDists.begin(),
 			inDists.end()));
@@ -263,7 +275,7 @@ static float getAlignmentIdentity(const Graph& g,
 		return max_identity;
 
 	vector<string> seqs(nbranches);
-	transform(first, last, seqs.begin(), bind(getSequence, &g, _1));
+	transform(first, last, seqs.begin(), boost::lambda::bind(getSequence, &g, _1));
 	for (unsigned i = 0; i < seqs.size(); i++) {
 		// Remove the overlapping sequence.
 		int n = seqs[i].size();
@@ -606,7 +618,7 @@ int main(int argc, char** argv)
 	if (opt::minCoverage > 0)
 		filterGraph(g);
 
-	if (opt::dot)
+	if (opt::bubbleGraph)
 		cout << "digraph bubbles {\n";
 
 	Bubbles bubbles = discoverBubbles(g);
@@ -619,7 +631,7 @@ int main(int argc, char** argv)
 	g_popped.erase(unique(g_popped.begin(), g_popped.end()),
 			g_popped.end());
 
-	if (opt::dot) {
+	if (opt::bubbleGraph) {
 		cout << "}\n";
 	} else {
 		for (vector<ContigID>::const_iterator it = g_popped.begin();
