@@ -60,10 +60,10 @@ PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Copyright 2014 Canada's Michael Smith Genome Science Centre\n";
 
 static const char USAGE_MESSAGE[] =
-"Usage: " PROGRAM " -k <kmer size> -k <kmer size>... -o <output_prefix> -S <path to scaffold file> [options]... <reads1> [reads2]...\n"
-"i.e. abyss-sealer -k90 -k80 -k70 -k60 -k50 -k40 -k30 -o test -S scaffold.fa read1.fa read2.fa\n\n"
+"Usage: " PROGRAM "-b <Bloom filter size> -k <kmer size> -k <kmer size>... -o <output_prefix> -S <path to scaffold file> [options]... <reads1> [reads2]...\n"
+"i.e. abyss-sealer -b20G -k90 -k80 -k70 -k60 -k50 -k40 -k30 -o test -S scaffold.fa read1.fa read2.fa\n\n"
 "Close gaps by using left and right flanking sequences of gaps as 'reads' for Konnector\n"
-"and performing multiple runs with each of the supplied K values..\n"
+"and performing multiple runs with each of the supplied K values.\n"
 "\n"
 " Options:\n"
 "\n"
@@ -71,16 +71,18 @@ static const char USAGE_MESSAGE[] =
 "  -S, --input-scaffold=FILE    load scaffold from FILE\n"
 "  -L, --flank-length=N         length of flanks to be used as pseudoreads [100]\n"
 "  -D, --flank-distance=N       distance of flank from gap [0]\n"
+"  -G, --max-gap-length=N       max gap size to fill in bp [800]; runtime increases\n"
+"                               exponentially with respect to this parameter\n"
 "  -j, --threads=N              use N parallel threads [1]\n"
 "  -k, --kmer=N                 the size of a k-mer\n"
-"  -b, --bloom-size=N           size of bloom filter [500M]\n"
+"  -b, --bloom-size=N           size of Bloom filter (e.g. '40G'). Required\n"
+"                               when not using pre-built Bloom filter(s)\n"
+"                               (-i option)\n"
 "  -B, --max-branches=N         max branches in de Bruijn graph traversal;\n"
 "                               use 'nolimit' for no limit [1000]\n"
 "  -d, --dot-file=FILE          write graph traversals to a DOT file\n"
 "  -e, --fix-errors             find and fix single-base errors when reads\n"
 "                               have no kmers in bloom filter [disabled]\n"
-"  -f, --min-frag=N             min fragment size in base pairs [0]\n"
-"  -F, --max-frag=N             max fragment size in base pairs [1000]\n"
 "  -i, --input-bloom=FILE       load bloom filter from FILE\n"
 "      --mask                   mask new and changed bases as lower case\n"
 "      --no-mask                do not mask bases [default]\n"
@@ -113,6 +115,22 @@ static const char USAGE_MESSAGE[] =
 "      --help                   display this help and exit\n"
 "      --version                output version information and exit\n"
 "\n"
+" Deprecated Options:\n"
+"\n"
+"  -f, --min-frag=N             min fragment size in base pairs\n"
+"  -F, --max-frag=N             max fragment size in base pairs\n"
+"\n"
+"   Note: --max-frag was formerly used to determine the maximum gap\n"
+"   size that abyss-sealer would attempt to close, according to the formula\n"
+"   max_gap_size = max_frag - 2 * flank_length, where flank_length is\n"
+"   deteremined by the -L option.  --max-frag is kept only for backwards\n"
+"   compatibility and is superceded by the more intuitive -G (--max-gap-length)\n"
+"   option. Similarly, --min-frag determines the minimum gap size to close,\n"
+"   according to the formula min_gap_size = min_frag - 2 * flank_length, where\n"
+"   a negative gap size indicates an overlap between gap flanks.  Normally the\n"
+"   user would not want to specify a minimum gap size and so it is recommended to\n"
+"   leave --min-frag unset.\n"
+"\n"
 "Report bugs to <" PACKAGE_BUGREPORT ">.\n";
 
 namespace opt {
@@ -123,6 +141,9 @@ namespace opt {
 	/** Distance of flank from gap. */
 	unsigned flankDistance = 0;
 
+	/** Max gap size to fill */
+	unsigned maxGapLength = 800;
+
 	/** scaffold file input. */
 	static string inputScaffold;
 
@@ -130,7 +151,7 @@ namespace opt {
 	static unsigned threads = 1;
 
 	/** The size of the bloom filter in bytes. */
-	size_t bloomSize = 500 * 1024 * 1024;
+	size_t bloomSize = 0;
 
 	/** The maximum count value of the BLoom filter. */
 	unsigned max_count = 2;
@@ -163,7 +184,7 @@ namespace opt {
 	unsigned minFrag = 0;
 
 	/** The maximum fragment size */
-	unsigned maxFrag = 1000;
+	unsigned maxFrag = 0;
 
 	/** Bloom filter input file */
 	static string inputBloomPath;
@@ -218,7 +239,7 @@ struct Counters {
 	size_t skipped;
 };
 
-static const char shortopts[] = "S:L:D:b:B:d:ef:F:i:Ij:k:lm:M:no:P:q:r:s:t:v";
+static const char shortopts[] = "S:L:D:b:B:d:ef:F:G:i:Ij:k:lm:M:no:P:q:r:s:t:v";
 
 enum { OPT_HELP = 1, OPT_VERSION };
 
@@ -228,6 +249,7 @@ static const struct option longopts[] = {
 	{ "input-scaffold",   required_argument, NULL, 'S' },
 	{ "flank-length",     required_argument, NULL, 'L' },
 	{ "flank-distance",   required_argument, NULL, 'D' },
+	{ "max-gap-length",   required_argument, NULL, 'G' },
 	{ "bloom-size",       required_argument, NULL, 'b' },
 	{ "max-branches",     required_argument, NULL, 'B' },
 	{ "dot-file",         required_argument, NULL, 'd' },
@@ -266,7 +288,7 @@ struct Coord
 {
 	int start;
 	int end;
-	
+
 	Coord() { }
 	Coord(int start, int end) : start(start), end(end) { }
 
@@ -701,6 +723,8 @@ int main(int argc, char** argv)
 			arg >> opt::flankLength; break;
 		  case 'D':
 			arg >> opt::flankDistance; break;
+		  case 'G':
+			arg >> opt::maxGapLength; break;
 		  case 'b':
 			opt::bloomSize = SIToBytes(arg); break;
 		  case 'B':
@@ -769,6 +793,14 @@ int main(int argc, char** argv)
 		}
 	}
 
+	/* translate --max-frag to --max-gap-length for backwards compatibility */
+	if (opt::maxFrag > 0) {
+		if ((int)opt::maxFrag < 2 * opt::flankLength)
+			opt::maxGapLength = 0;
+		else
+			opt::maxGapLength = opt::maxFrag - 2 * opt::flankLength;
+	}
+
 	if (opt::inputScaffold.empty()) {
 		cerr << PROGRAM ": missing mandatory option `-S'\n";
 		die = true;
@@ -779,14 +811,36 @@ int main(int argc, char** argv)
 		die = true;
 	}
 
+	if (opt::bloomFilterPaths.size() < opt::kvector.size()
+		&& opt::bloomSize == 0)
+	{
+		cerr << PROGRAM ": missing mandatory option `-b' (Bloom filter size)\n"
+			<< "Here are some guidelines for sizing the Bloom filter:\n"
+			<< "  * E. coli (~5 Mbp genome), 615X coverage: -b500M\n"
+			<< "  * S. cerevisiae (~12 Mbp genome), 25X coverage: -b500M\n"
+			<< "  * C. elegans (~100 Mbp genome), 89X coverage: -b1200M\n"
+			<< "  * H. sapiens (~3 Gbp genome), 71X coverage: -b40G\n";
+		die = true;
+	}
+
 	if (opt::outputPrefix.empty()) {
 		cerr << PROGRAM ": missing mandatory option `-o'\n";
 		die = true;
 	}
 
-	if (argc - optind < 1) {
+	if (opt::bloomFilterPaths.size() > opt::kvector.size()) {
+		cerr << PROGRAM ": you must specify a k-mer size (-k) for each Bloom "
+			" filter file (-i)\n";
+		die = true;
+	} else if (opt::bloomFilterPaths.size() < opt::kvector.size()
+		&& argc - optind < 1) {
 		cerr << PROGRAM ": missing input file arguments\n";
 		die = true;
+	} else if (opt::bloomFilterPaths.size() == opt::kvector.size()
+		&& argc - optind > 0) {
+		cerr << PROGRAM ": input FASTA/FASTQ args should be omitted when using "
+			"pre-built Bloom filters (-i) for all k-mer sizes\n";
+		die = true;
 	}
 
 	if (die) {
@@ -806,8 +860,6 @@ int main(int argc, char** argv)
 	seqanTests();
 #endif
 
-	assert(opt::bloomSize > 0);
-
 	ofstream dotStream;
 	if (!opt::dotPath.empty()) {
 		if (opt::verbose)
@@ -848,7 +900,7 @@ int main(int argc, char** argv)
 	ConnectPairsParams params;
 
 	params.minMergedSeqLen = opt::minFrag;
-	params.maxMergedSeqLen = opt::maxFrag;
+	params.maxMergedSeqLen = opt::maxGapLength + 2 * opt::flankLength;
 	params.maxPaths = opt::maxPaths;
 	params.maxBranches = opt::maxBranches;
 	params.maxPathMismatches = opt::maxMismatches;
@@ -893,12 +945,12 @@ int main(int argc, char** argv)
 		map<FastaRecord, Gap>::iterator read2_it;
 
 		string read1OutputPath(opt::outputPrefix);
-		read1OutputPath.append("_flanks_1.fq");
+		read1OutputPath.append("_flanks_1.fa");
 		ofstream read1Stream(read1OutputPath.c_str());
 		assert_good(read1Stream, read1OutputPath);
 
 		string read2OutputPath(opt::outputPrefix);
-		read2OutputPath.append("_flanks_2.fq");
+		read2OutputPath.append("_flanks_2.fa");
 		ofstream read2Stream(read2OutputPath.c_str());
 		assert_good(read2Stream, read2OutputPath);
 
@@ -934,7 +986,7 @@ int main(int argc, char** argv)
 		BloomFilter* bloom;
 		CascadingBloomFilter* cascadingBloom = NULL;
 
-		if (!opt::bloomFilterPaths.empty() && i <= opt::bloomFilterPaths.size()) {
+		if (!opt::bloomFilterPaths.empty() && i < opt::bloomFilterPaths.size()) {
 
 			temp = "Loading bloom filter from `" + opt::bloomFilterPaths.at(i) + "'...\n";
 			printLog(logStream, temp);
@@ -963,6 +1015,12 @@ int main(int argc, char** argv)
 			bloom = &cascadingBloom->getBloomFilter(opt::max_count - 1);
 		}
 
+		assert(bloom != NULL);
+
+		if (opt::verbose)
+			cerr << "Bloom filter FPR: " << setprecision(3)
+				<< 100 * bloom->FPR() << "%\n";
+
 		DBGBloom<BloomFilter> g(*bloom);
 
 		temp = "Starting K run with k = " + IntToString(opt::k) + "\n";
